@@ -218,10 +218,6 @@ class ChildIOException(WorkflowError):
         super().__init__(msg, lineno=lineno, snakefile=snakefile, rule=rule)
 
 
-class MissingOutputException(RuleException):
-    pass
-
-
 class IOException(RuleException):
     def __init__(self, prefix, rule, files, include=None, lineno=None, snakefile=None):
         message = (
@@ -238,6 +234,16 @@ class IOException(RuleException):
         )
 
 
+class MissingOutputException(RuleException):
+    def __init__(
+        self, message=None, include=None, lineno=None, snakefile=None, rule=None
+    ):
+        message = "Job completed successfully, but some output files are missing. {}".format(
+            message
+        )
+        super().__init__(message, include, lineno, snakefile, rule)
+
+
 class MissingInputException(IOException):
     def __init__(self, rule, files, include=None, lineno=None, snakefile=None):
         msg = "Missing input files"
