@@ -61,11 +61,11 @@ class JobScheduler:
         drmaa=None,
         drmaa_log_dir=None,
         kubernetes=None,
-        kubernetes_envvars=None,
         container_image=None,
         tibanna=None,
         tibanna_sfn=None,
         precommand="",
+        tibanna_config=False,
         jobname=None,
         quiet=False,
         printreason=False,
@@ -98,7 +98,11 @@ class JobScheduler:
         self.max_jobs_per_second = max_jobs_per_second
         self.keepincomplete = keepincomplete
 
-        self.resources = dict(self.workflow.global_resources)
+        self.global_resources = {
+            name: (sys.maxsize if res is None else res)
+            for name, res in workflow.global_resources.items()
+        }
+        self.resources = dict(self.global_resources)
 
         use_threads = (
             force_use_threads
@@ -220,7 +224,6 @@ class JobScheduler:
                 workflow,
                 dag,
                 kubernetes,
-                kubernetes_envvars,
                 container_image=container_image,
                 printreason=printreason,
                 quiet=quiet,
@@ -249,6 +252,7 @@ class JobScheduler:
                 cores,
                 tibanna_sfn,
                 precommand=precommand,
+                tibanna_config=tibanna_config,
                 container_image=container_image,
                 printreason=printreason,
                 quiet=quiet,
@@ -507,7 +511,7 @@ Problem", Akcay, Li, Xu, Annals of Operations Research, 2012
                 return [c_j * y_j for c_j, y_j in zip(c, y)]
 
             b = [
-                self.resources[name] for name in self.workflow.global_resources
+                self.resources[name] for name in self.global_resources
             ]  # resource capacities
 
             while True:
@@ -548,12 +552,12 @@ Problem", Akcay, Li, Xu, Annals of Operations Research, 2012
 
             solution = [job for job, sel in zip(jobs, x) if sel]
             # update resources
-            for name, b_i in zip(self.workflow.global_resources, b):
+            for name, b_i in zip(self.global_resources, b):
                 self.resources[name] = b_i
             return solution
 
     def calc_resource(self, name, value):
-        gres = self.workflow.global_resources[name]
+        gres = self.global_resources[name]
         if value > gres:
             if name == "_cores":
                 name = "threads"
@@ -569,15 +573,13 @@ Problem", Akcay, Li, Xu, Annals of Operations Research, 2012
     def rule_weight(self, rule):
         res = rule.resources
         return [
-            self.calc_resource(name, res.get(name, 0))
-            for name in self.workflow.global_resources
+            self.calc_resource(name, res.get(name, 0)) for name in self.global_resources
         ]
 
     def job_weight(self, job):
         res = job.resources
         return [
-            self.calc_resource(name, res.get(name, 0))
-            for name in self.workflow.global_resources
+            self.calc_resource(name, res.get(name, 0)) for name in self.global_resources
         ]
 
     def job_reward(self, job):
